@@ -1,6 +1,9 @@
+import six
+
 from funk.util import arguments_str
 
-__all__ = ['any_value', 'is_a', 'has_attr', 'equal_to', 'not_', 'all_of', 'any_of', 'is_']
+__all__ = ['any_value', 'is_a', 'has_attr', 'equal_to', 'not_', 'all_of',
+           'any_of', 'is_', 'contains_exactly']
 
 class Matcher(object):
     pass
@@ -29,7 +32,8 @@ class IsA(Matcher):
         return self._describe(self._type)
         
     def _describe(self, value_type):
-        if value_type.__module__ == "__builtin__":
+        module = value_type.__module__
+        if (six.PY2 and module == "__builtin__") or (six.PY3 and module == "builtins"):
             module_prefix = ''
         else:
             module_prefix = '%s.' % value_type.__module__
@@ -41,11 +45,11 @@ def is_a(type_):
 class HasAttr(Matcher):
     def __init__(self, attributes):
         self._attributes = {}
-        for key, value in attributes.iteritems():
+        for key, value in six.iteritems(attributes):
             self._attributes[key] = self._to_matcher(value)
         
     def matches(self, value, mismatch_output):
-        for key in self._attributes:
+        for key in sorted(six.iterkeys(self._attributes)):
             if not hasattr(value, key):
                 mismatch_output.append("value was missing attribute: %s" % key)
                 return False
@@ -148,3 +152,48 @@ class Is(Matcher):
         
 def is_(value):
     return Is(value)
+
+def _is_iterable(value):
+    try:
+        iter(value)
+        return True
+    except TypeError:
+        return False
+
+class ContainsExactly(Matcher):
+    def __init__(self, matchers):
+        self._matchers = matchers
+        
+    def matches(self, original, mismatch_output):
+        if not _is_iterable(original):
+            mismatch_output.append("was not iterable %s" % (self._got_str(original), ))
+            return False
+        other = list(original)
+        for matcher in self._matchers:
+            matched = False
+            for index, element in enumerate(other):
+                if matcher.matches(element, []):
+                    matched = True
+                    other.pop(index)
+                    break
+            if not matched:
+                mismatch_output.append("iterable did not contain element: %s %s" % (matcher, self._got_str(original)))
+                return False
+        if len(other) > 0:
+            mismatch_output.append("iterable contained extra elements: %s %s" % (", ".join(map(repr, other)), self._got_str(original)))
+            return False
+        return True
+    
+    def _got_str(self, original):
+        return "(got: %s)" % (repr(original), )
+    
+    def __str__(self):
+        return "<iterable containing exactly: %s>" % (", ".join(map(str, self._matchers)), )
+
+def contains_exactly(*matchers):
+    return ContainsExactly(map(to_matcher, matchers))
+
+def to_matcher(value):
+    if isinstance(value, Matcher):
+        return value
+    return equal_to(value)
