@@ -8,6 +8,7 @@ from funk.matchers import not_
 from funk.matchers import all_of
 from funk.matchers import any_of
 from funk.matchers import is_
+from funk.matchers import contains_exactly
 
 def test_any_value_matches_everything():
     assert any_value().matches("foo", [])
@@ -25,9 +26,10 @@ def test_any_value_does_not_write_to_mismatch_output():
     assert not mismatch_output
 
 def test_is_a_matches_on_type():
-    assert is_a(type).matches(basestring, [])
-    assert is_a(basestring).matches("foo", [])
-    assert not is_a(basestring).matches(4, [])
+    assert is_a(type).matches(str, [])
+    assert is_a(str).matches("foo", [])
+    assert is_a(object).matches("foo", [])
+    assert not is_a(str).matches(4, [])
 
 def test_is_a_str_shows_type():
     class SomeClass(object):
@@ -36,15 +38,15 @@ def test_is_a_str_shows_type():
 
 def test_is_a_does_not_write_to_mismatch_output_if_it_matches():
     mismatch_output = []
-    assert is_a(type).matches(basestring, mismatch_output)
-    assert is_a(basestring).matches("foo", mismatch_output)
+    assert is_a(type).matches(str, mismatch_output)
+    assert is_a(str).matches("foo", mismatch_output)
     assert not mismatch_output
     
 def test_is_a_writes_type_of_value_if_it_does_not_match():
     class SomeClass(object):
         pass
     mismatch_output = []
-    is_a(basestring).matches(SomeClass(), mismatch_output)
+    is_a(str).matches(SomeClass(), mismatch_output)
     assert_equals(["got <value of type: test.funk.test_matchers.SomeClass>"], mismatch_output)
 
 def test_is_a_does_not_show_module_if_type_is_builtin():
@@ -73,7 +75,7 @@ def test_has_attr_can_use_matchers():
     assert not matcher.matches(Person(42), [])
     
 def test_has_attr_str_contains_all_attributes():
-    assert_equals(str(has_attr(what="else", key="word")), "<value with attributes: what='else', key='word'>")
+    assert_equals(str(has_attr(what="else", key="word")), "<value with attributes: key='word', what='else'>")
 
 def test_has_attr_does_not_write_to_mismatch_output_if_values_match():
     class Rectangle(object):
@@ -92,7 +94,7 @@ def test_has_attr_describes_actual_attributes_if_present():
     mismatch_output = []
     has_attr(width=20, height=40).matches(Rectangle(30, 50), mismatch_output)
     
-    assert_equals(["got <value with attribute: width=30>"], mismatch_output)
+    assert_equals(["got <value with attribute: height=50>"], mismatch_output)
     
 def test_has_attr_will_describe_missing_attributes():
     mismatch_output = []
@@ -270,3 +272,49 @@ def test_is_writes_str_of_non_matching_values_to_mismatch_output():
     mismatch_output = []
     is_("{}").matches("Hello!", mismatch_output)
     assert_equals(["got: Hello!"], mismatch_output)
+
+def test_contains_exactly_empty_array_only_matches_empty_collections():
+    assert contains_exactly().matches([], [])
+    assert contains_exactly().matches((), [])
+    assert not contains_exactly().matches([None], [])
+    assert not contains_exactly().matches((None, ), [])
+    assert not contains_exactly().matches(None, [])
+    
+def test_contains_exactly_matches_any_collection_with_only_those_elements_in_any_order():
+    assert contains_exactly(equal_to(42), equal_to(9)).matches([42, 9], [])
+    assert contains_exactly(equal_to(42), equal_to(9)).matches(set([42, 9]), [])
+    assert contains_exactly(equal_to(42), equal_to(9)).matches([9, 42], [])
+    assert not contains_exactly(equal_to(42), equal_to(9)).matches([42], [])
+    assert not contains_exactly(equal_to(42), equal_to(9)).matches([9], [])
+    assert not contains_exactly(equal_to(42), equal_to(9)).matches([], [])
+    assert not contains_exactly(equal_to(42), equal_to(9)).matches([42, 9, 1], [])
+
+def test_contains_exactly_describes_all_sub_matchers():
+    assert_equals(str(contains_exactly(equal_to(42), equal_to(9))), "<iterable containing exactly: 42, 9>")
+
+def test_contains_exactly_does_not_write_to_mismatch_output_if_it_matches():
+    mismatch_output = []
+    assert contains_exactly().matches([], mismatch_output)
+    assert contains_exactly().matches((), mismatch_output)
+    assert contains_exactly(equal_to(42), equal_to(9)).matches([42, 9], mismatch_output)
+    assert contains_exactly(equal_to(42), equal_to(9)).matches([9, 42], mismatch_output)
+    assert not mismatch_output
+
+def test_contains_exactly_describes_missing_element_in_mismatch_output():
+    mismatch_output = []
+    contains_exactly(equal_to(42), equal_to(9)).matches([42], mismatch_output)
+    assert_equals(["iterable did not contain element: 9 (got: [42])"], mismatch_output)
+
+def test_contains_exactly_writes_not_iterable_to_mismatch_output_if_not_passed_an_iterable():
+    mismatch_output = []
+    contains_exactly(equal_to(42), equal_to(9)).matches(None, mismatch_output)
+    assert_equals(["was not iterable (got: None)"], mismatch_output)
+
+def test_contains_exactly_writes_extra_elements_to_mismatch_output():
+    mismatch_output = []
+    contains_exactly(equal_to(42), equal_to(9)).matches([42, 1, 9, "eggs"], mismatch_output)
+    assert_equals(["iterable contained extra elements: 1, 'eggs' (got: [42, 1, 9, 'eggs'])"], mismatch_output)
+
+def test_contains_exactly_converts_non_matcher_elements_to_matchers_using_equal_to():
+    assert contains_exactly(42, 9).matches([42, 9], [])
+    assert not contains_exactly(42, 9).matches([42, 9, 1], [])
